


var FlipView = null;

(function($) {
    
FlipView = Class.extend({
	/**
	 * Konstruktor
	 */
    init: function (options) {
        this.defaults = {
            containerId: "#Rows",
            itemClass: ".row",
            spaceBetweenItems: 120,
            itemWidth: 924,
            itemHeight: 584,
            dragTolerance: 60,
            onSlideFinished: null,
	    initialIndex:null,
	    animationType: 'linear',
	    dragDisable: false
        };

        this.options = $.extend(this.defaults, options);
        this.container = $(this.options.containerId);

        this.items = this.container.find(this.options.itemClass);
        this.itemsTotal = this.items.length;

        this.currentItemIndex = 0;

        this.inDrag = false;

        this.initialize();
    },

	/**
	 * initialize
	 * Setzt die benötigten Größen und Abstände
	 */
    initialize: function () {
        this.container.css("width", this.itemsTotal * this.options.itemWidth + this.itemsTotal * this.options.spaceBetweenItems);
        this.container.css("height", this.options.itemHeight);
        this.items.css("marginRight", this.options.spaceBetweenItems);

        this.addDrag();

        if (this.options.initialIndex != null) {
        	this.slideTo(this.options.initialIndex, false, true);
        }
    },

	/**
	 * getCurrentItemIndex
	 * Setzt die benötigten Größen und Abstände
	 */
    getCurrentItemIndex: function () {
    	return this.currentItemIndex;
    },

    /**
     * getItemsTotal
     * Gibt die Anzahl der verfügbaren Slides wieder
     */
    getItemsTotal: function () {
    	return this.itemsTotal;
    },

	/**
	 * addDrag
	 * Erstellt die Drag-Funktionalität mithilfe von jQuery
	 */
    addDrag: function () {
        var self = this;
	
        this.container.draggable({
            axis: 'x',
            start: function (event, ui) {
            	
            },
            drag: function (event, ui) {
                self.onDrag(event, ui);
            },
            stop: function (event, ui) {
            	self.onDragStop(event, ui);
            }
        });
	
	if(this.options.dragDisable) {
	    this.container
		.draggable({ disabled: true })
		.css({'opacity': '1'});
	    return;
	}
    },

	/**
	 * onDrag
	 * Wird während des dragging ausgeführt
	 */
    onDrag: function (event, ui) {
    	// Erst wenn die Position sich auf der X Achse um mindestens 10 Pixel verschoben hat 
    	// befinden wir uns im Drag-Modus, da sonst der Touchevent nicht ausgeführt werden kann
    	if (Math.abs(ui.position.left - ui.originalPosition.left) >= 10) {
    		this.inDrag = true;
    	}
    },

	/**
	 * onDragStop
	 * Wird nach dem dragging ausgeführt
	 */
    onDragStop: function (event, ui) {
        this.snapToItem(ui);
    },

	/**
	 * snapToItem
	 * Entscheided ob ein Slide durchgeführt wird und in welche Richtung
	 */
    snapToItem: function (ui) {
        // Drag Toleranz überschritten?
        if (Math.abs(ui.originalPosition.left - ui.position.left) >= this.options.dragTolerance) {
            if (ui.position.left < ui.originalPosition.left) {
                this.slideToRight();
            } else {
                this.slideToLeft();
            }
        } else {
            this.slideTo(this.currentItemIndex, true, true);
        }
    },

	/**
	 * slideToLeft
	 * Slide nach links
	 */
    slideToLeft: function () {
        if (this.currentItemIndex > 0) {
            this.currentItemIndex--;
            this.slideTo(this.currentItemIndex);
        } else {
        	this.slideTo(0, true, true);
        }
    },

	/**
	 * slideToRight
	 * Slide nach rechts
	 */
    slideToRight: function () {
        if (this.currentItemIndex < this.itemsTotal - 1) {
            this.currentItemIndex++;
            this.slideTo(this.currentItemIndex);
        } else {
        	this.slideTo(this.itemsTotal - 1, true, true);
        }
    },

	/**
	 * slideTo
	 * Zu einem bestimmten index animieren
	 * default ist mit animation. Setzte man den 2. Paramter auf false dann wird keine Animation durchgeführt
	 */
    slideTo: function (index) {
    	if (index >= this.itemsTotal || index < 0 ) {
    		this.inDrag = false;
    		return;
    	}
    	this.currentItemIndex = index;

    	//Optional Arguments
    	var animate = true;
    	if (arguments.length > 1) {
    		var animate = arguments[1];
    	}
    	var withoutEvent = false;
    	if (arguments.length > 2) {
    		var withoutEvent = arguments[2];
    	}

    	var self = this;
    	var slideToPos = index * (this.options.itemWidth + this.options.spaceBetweenItems);

    	if (animate) {
    	    this.container.animate({ left: -slideToPos }, 200, this.options.animationType, function () {
    	        self.inDrag = false;
    			if (self.options.onSlideFinished != null && !withoutEvent) {
    				self.options.onSlideFinished(self.currentItemIndex);
    			}
    		});
    	} else {
    		this.container.css("left", -slideToPos);
    		if (this.options.onSlideFinished != null && !withoutEvent) {
    			this.options.onSlideFinished(this.currentItemIndex);
    		}
    		this.inDrag = false;
    	}
    },

	/**
	 * isInDrag
	 * Gibt Auskunft darüber ob sich das Grid im Drag-Modus befindet
	 */
    isInDrag: function () {
    	return this.inDrag;
    },

	/**
	 * destruct
	 * Nicht benötigte Elemente abräumen
	 */
    destruct: function () {
    	delete this;
    }


    /*
    hideInvisible: function (position) {
        if (position.left < 0) {
            var pos = Math.abs(position.left);
            var itemsInvisibleLeft = Math.floor(pos / (this.options.itemWidth + this.options.spaceBetweenItems));
            console.log(itemsInvisibleLeft);
            if (itemsInvisibleLeft > 0) {
                this.items.each(function (index, elem) {
                    if (index < itemsInvisibleLeft) {
                        $(elem).hide();
                    }
                });
            }
        }
    }*/
});

})(jQuery);